<?php

namespace SGalinski\SgNews\TCA;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (http://www.sgalinski.de)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Backend\Form\FormEngine;
use TYPO3\CMS\Core\Database\DatabaseConnection;
use TYPO3\CMS\Core\DataHandling\DataHandler;
use TYPO3\CMS\Core\Resource\ResourceFactory;
use TYPO3\CMS\Core\SingletonInterface;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\PathUtility;
use TYPO3\CMS\Lang\LanguageService;

/**
 * This class contains methods for usage within the TCA forms.
 */
class TcaProvider implements SingletonInterface {

	/**
	 * @var \TYPO3\CMS\Extbase\Object\ObjectManagerInterface
	 */
	protected $objectManager;

	/**
	 * Constructor
	 */
	public function __construct() {
		$this->objectManager = GeneralUtility::makeInstance('TYPO3\CMS\Extbase\Object\ObjectManager');
	}

	/**
	 * Hook method for calculating the title for articles and copying the subtitle into the nav_title field
	 *
	 * @param DataHandler $tceMain
	 * @return void
	 */
	public function processDatamap_afterAllOperations(DataHandler $tceMain) {
		/** @var DatabaseConnection $database */
		$database = $GLOBALS['TYPO3_DB'];

		foreach ($tceMain->datamap as $table => $tablesData) {
			if (!in_array($table, ['pages', 'pages_language_overlay'])) {
				continue;
			}

			foreach ($tablesData as $identity => $tableData) {
				if (strpos($identity, 'NEW') !== FALSE) {
					$identity = $tceMain->substNEWwithIDs[$identity];
				}

				$translationRow = [];
				$pagesIdentity = (int) $identity;
				if ($table === 'pages_language_overlay') {
					$translationRow = $database->exec_SELECTgetSingleRow(
						'pid, subtitle', 'pages_language_overlay', 'uid = ' . (int) $identity
					);
					$pagesIdentity = (int) $translationRow['pid'];
				}

				$row = $database->exec_SELECTgetSingleRow(
					'doktype, subtitle, lastUpdated, tx_sgnews_highlighted', 'pages', 'uid = ' . $pagesIdentity
				);

				// only articles
				$row['doktype'] = (int) $row['doktype'];
				if (!is_array($row) || !count($row) || !in_array($row['doktype'], [116])) {
					continue;
				}

				$prefix = '';
				if ($row['tx_sgnews_highlighted'] >= 1) {
					$prefix = 'Top - ';
				}

				$subtitle = $row['subtitle'];
				if ($table === 'pages_language_overlay') {
					$subtitle = $translationRow['subtitle'];
				}
				$calculatedTitle = $prefix . strftime('%Y-%m-%d', $row['lastUpdated']) . ' - ' . $subtitle;

				$updateFields = ['nav_title' => $subtitle, 'title' => trim($calculatedTitle)];
				$database->exec_UPDATEquery($table, 'uid = ' . $identity, $updateFields);
			}
		}
	}

	/**
	 * Allows the selection of the coordinates from the first attached image of the configured sourceFieldName like
	 * media. The selection is added into a hidden field by javascript that is saved by TYPO3.
	 *
	 * @param array $parameters
	 * @param FormEngine $formEngine
	 * @return string
	 */
	public function coordinatePicker(array $parameters, FormEngine $formEngine) {
		// add Jcrop
		$requestHost = GeneralUtility::getIndpEnv('TYPO3_REQUEST_HOST');
		$jsFile = GeneralUtility::getFileAbsFileName('EXT:sg_news/Resources/Public/Contrib/Jcrop/jquery.Jcrop.min.js');
		$jsFile = $requestHost . '/' . PathUtility::stripPathSitePrefix($jsFile);
		$cssFile = 'EXT:sg_news/Resources/Public/Contrib/Jcrop/jquery.Jcrop.min.css';
		$cssFile = GeneralUtility::getFileAbsFileName($cssFile);
		$cssFile = $requestHost . '/' . PathUtility::stripPathSitePrefix($cssFile);
		$formEngine->additionalCode_pre['rsNewsCoordinatePicker'] = '
			<script src="' . $jsFile . '" type="text/javascript"></script>
			<link rel="stylesheet" href="' . $cssFile . '" type="text/css" />
		';

		// add hidden form field content
		$hash = uniqid('tx-sgnews-');
		$field = $parameters['field'];
		$column = 'data[pages][' . $parameters['row']['uid'] . '][' . $field . ']';
		$content = '<input id="' . $hash . '-input" name="' . $column .
			'" value="' . $parameters['row'][$field] . '" type="hidden" />';

		// add jcrop instance javascript
		$coordinates = GeneralUtility::intExplode(',', $parameters['row'][$field], TRUE);
		$width = (int) $parameters['fieldConf']['config']['parameters']['width'];
		$height = (int) $parameters['fieldConf']['config']['parameters']['height'];
		$coordinates = (!count($coordinates) ? [0, 0, $width, $height] : $coordinates);
		$aspectRatio = $parameters['fieldConf']['config']['parameters']['aspectRatio'];
		$formEngine->additionalJS_pre['rsNewsCoordinatePicker' . $hash] = '
			(function($) {
				$(document).ready(function() {
					var writeCoords = function (coordinates) {
						var result = coordinates.x + "," + coordinates.y + "," + coordinates.x2 + "," +
						 	coordinates.y2 + "," + coordinates.w + "," + coordinates.h;
						$("#' . $hash . '-input").val(result);
					};

					$("#' . $hash . '-source").Jcrop({
						onChange: writeCoords,
						onSelect: writeCoords,
						boxWidth: 800,
						boxHeight: 600,
						aspectRatio: ' . htmlspecialchars($aspectRatio) . ',
						minSize: [' . $width . ',' . $height . '],
						setSelect: [' . implode(',', $coordinates) . ']
					});
				});
			})(TYPO3.jQuery);
		';

		// add image for jcrop
		$sourceFieldName = $parameters['fieldConf']['config']['parameters']['sourceFieldName'];
		$firstImageReferenceId = GeneralUtility::intExplode(',', $parameters['row'][$sourceFieldName])[0];
		try {
			$fileReference = ResourceFactory::getInstance()->getFileReferenceObject($firstImageReferenceId);
			if ($fileReference) {
				$file = $requestHost . '/' . $fileReference->getPublicUrl();
				$content .= '<img src="' . $file . '" alt="" id="' . $hash . '-source" />';
			}
		} catch (\Exception $exception) {
			/** @var LanguageService $languageService */
			$languageService = $GLOBALS['LANG'];
			$languageService->includeLLFile('EXT:sg_news/Resources/Private/Language/locallang_backend.xlf');
			$content = '<strong>' . $languageService->getLL('coordinatePicker.missingImage') . '</strong>';
		}

		return $content;
	}
}

?>
